/**
 * 
 */
package cmis.junit.examples;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.UUID;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.oasis_open.docs.ns.cmis.core._200901.CmisAllowableActionsType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisChangeEventType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisObjectType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertiesType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisProperty;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyBoolean;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyBooleanDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyDateTime;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyDateTimeDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyId;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyIdDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyInteger;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyIntegerDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyString;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyStringDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyUri;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyUriDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisQueryType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisRepositoryCapabilitiesType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisRepositoryInfoType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisTypeDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisTypeDocumentDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisTypeFolderDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisTypePolicyDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisTypeRelationshipDefinitionType;
import org.oasis_open.docs.ns.cmis.core._200901.EnumBaseObjectTypeIds;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityACL;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityChanges;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityContentStreamUpdates;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityJoin;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityQuery;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCardinality;
import org.oasis_open.docs.ns.cmis.core._200901.EnumContentStreamAllowed;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesBase;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesDocument;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesFolder;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesPolicy;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesRelationship;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertyType;
import org.oasis_open.docs.ns.cmis.core._200901.EnumTypeOfChanges;
import org.oasis_open.docs.ns.cmis.core._200901.EnumUpdatability;
import org.oasis_open.docs.ns.cmis.restatom._200901.CmisAtomChildren;
import org.oasis_open.docs.ns.cmis.restatom._200901.CmisUriTemplateType;
import org.oasis_open.docs.ns.cmis.restatom._200901.EnumCollectionType;
import org.oasis_open.docs.ns.cmis.restatom._200901.EnumLinkRelations;
import org.oasis_open.docs.ns.cmis.restatom._200901.EnumUriTemplateType;
import org.w3._2005.atom.AtomDateConstruct;
import org.w3._2005.atom.AtomPersonConstruct;
import org.w3._2005.atom.AtomTextConstruct;
import org.w3._2005.atom.Content;
import org.w3._2005.atom.EntryType;
import org.w3._2005.atom.FeedType;
import org.w3._2005.atom.Id;
import org.w3._2005.atom.Link;
import org.w3._2007.app.AppCollectionType;
import org.w3._2007.app.AppServiceType;
import org.w3._2007.app.AppWorkspaceType;

/**
 * @author abrown
 * 
 */
public class ExampleTest extends TestCase {

	private static final String _CORE = "org.oasis_open.docs.ns.cmis.core._200901";
	private static final String _MSG = "org.oasis_open.docs.ns.cmis.messaging._200901";
	private static final String _RA = "org.oasis_open.docs.ns.cmis.restatom._200901";
	private static final String _ATOM = "org.w3._2005.atom";
	private static final String _APP = "org.w3._2007.app";

	private static BigDecimal _CMIS_VERSION = BigDecimal.valueOf(0.62);

	private static final String AUTHOR_NAME = "Al Brown";
	private static final String AUTHOR_EMAIL = "albertcbrown@us.ibm.com";
	private static final String AUTHOR_URI = "http://www.ibm.com/";

	private static final String CS_LENGTH = "4123";
	private static final String CS_MIMETYPE = "text/plain";

	private static final String URL_BASE = "http://cmisexample.oasis-open.org/rep1/";

	public static final String MIMETYPE_QUERY = "application/cmisquery+xml";
	public static final String MIMETYPE_ATOM = "application/atom+xml";
	public static final String MIMETYPE_ATOMENTRY = "application/atom+xml;type=entry";
	public static final String MIMETYPE_ALLOWABLEACTIONS = "application/cmis+xml;type=allowableActions";
	public static final String MIMETYPE_ATOMFEED = "application/atom+xml;type=feed";
	public static final String MIMETYPE_TYPE = "application/cmis+xml;type=type";
	public static final String MIMETYPE_APPSERVICE = "application/atomsvc+xml";
	public static final String MIMETYPE_TEXT = "text/plain";
	public static final String MIMETYPE_HTML = "text/html";
	public static final String MIMETYPE_CMISTREE = "application/cmistree+xml";
	private static String _PRODUCT_NAME = "CMIS Prototype for VendorX";
	private static String _PRODUCT_VERSION = _CMIS_VERSION.toString();

	private org.oasis_open.docs.ns.cmis.core._200901.ObjectFactory coreFactory = new org.oasis_open.docs.ns.cmis.core._200901.ObjectFactory();
	private org.oasis_open.docs.ns.cmis.restatom._200901.ObjectFactory raFactory = new org.oasis_open.docs.ns.cmis.restatom._200901.ObjectFactory();
	private org.w3._2005.atom.ObjectFactory atomFactory = new org.w3._2005.atom.ObjectFactory();
	private org.w3._2007.app.ObjectFactory appFactory = new org.w3._2007.app.ObjectFactory();

	private static final String JAXB_PACKAGE_LIST = _CORE + ":" + _MSG + ":"
			+ _ATOM + ":" + _APP + ":" + _RA;

	private static Marshaller marshaller;
	private static JAXBContext jaxbContext;

	public void test01AllowableActions() {
		CmisAllowableActionsType aa = getAllowableActions(EnumBaseObjectTypeIds.CMIS_DOCUMENT);

		// write it
		JAXBElement<CmisAllowableActionsType> element = coreFactory
				.createAllowableActions(aa);
		writeOutput("AllowableActions", element);
	}

	public EntryType getAtomEntryType(String objectId, EnumBaseObjectTypeIds bot,
			String typeid, boolean checkedout, boolean allowableActions,
			boolean changeEntry, String name) {
		EntryType e = new EntryType();

		// <atom:author xml:base="http://tempuri.org" xml:lang="">
		// <atom:name>Al Brown</atom:name>
		// </atom:author>
		e.getAuthor().add(getAtomPerson(AUTHOR_NAME, AUTHOR_EMAIL, AUTHOR_URI));

		// <!-- atom:content -->
		// <atom:content src="http://www.cmis.org/rep1/media/document-entry">
		// </atom:content>

		Content c = new Content();
		c.setSrc(URL_BASE + objectId);
		e.setContent(c);

		// <!-- IRI to document entry resource -->
		// <atom:id>http://www.cmis.org/rep1/document-entry</atom:id>
		Id id = new Id();
		id.setContent("urn:uuid:" + objectId);
		e.setId(id);

		// <!-- atom links -->
		// <atom:link rel='self' href="http://www.cmis.org/rep1/document-entry"
		// />
		e.getLink()
				.add(
						getAtomLink("self", URL_BASE + objectId, null, null,
								null, null));

		// <atom:link rel="edit" href="http://www.cmis.org/rep1/document-entry"
		// />
		e.getLink()
				.add(
						getAtomLink("edit", URL_BASE + objectId, null, null,
								null, null));

		// <atom:link rel="cmis-allowableactions"
		// href="http://www.cmis.org/rep1/document-entry/actions" />
		e.getLink().add(
				getAtomLink("allowableactions", URL_BASE + objectId
						+ "/allowableactions", MIMETYPE_ALLOWABLEACTIONS, null,
						null, null));

		// <atom:link rel="cmis-type" href="http://www.cmis.org/rep1/type/email"
		// />
		e.getLink().add(
				getAtomLink("type", URL_BASE + objectId + "/type",
						MIMETYPE_ATOMENTRY, null, null, null));

		// <!-- createdDate -->
		// <atom:published>2001-12-31T12:00:00</atom:published>
		e.setPublished(getAtomDateConstruct(new Date()));

		// <atom:summary>
		// <!-- auto-generated HTML table of properties -->
		// </atom:summary>
		e.setSummary(getAtomText("HTML summary of Entry  " + objectId, "html"));

		// <atom:title>Document Entry example</atom:title>
		e.setTitle(getAtomText(name, "text"));

		// <!-- lastModifiedDate -->
		// <atom:updated>2001-12-31T12:00:00</atom:updated>
		e.setUpdated(getAtomDateConstruct(new Date()));

		CmisObjectType cot = new CmisObjectType();
		CmisPropertiesType cpt = new CmisPropertiesType();

		// Bools
		cpt.getProperty().add(
				getPropertyBoolean(EnumPropertiesDocument.CMIS_IS_IMMUTABLE
						.value(), false));

		// Dates
		cpt.getProperty().add(
				getPropertyDate(EnumPropertiesBase.CMIS_CREATION_DATE.value(),
						new Date()));
		cpt.getProperty().add(
				getPropertyDate(EnumPropertiesBase.CMIS_LAST_MODIFICATION_DATE
						.value(), new Date()));

		// Ids
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesBase.CMIS_OBJECT_ID.value(),
						objectId));
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesBase.CMIS_OBJECT_TYPE_ID.value(),
						typeid));
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesBase.CMIS_BASE_TYPE_ID.value(), bot
						.value()));

		// Integers

		// Strings
		cpt.getProperty().add(
				getPropertyString(EnumPropertiesBase.CMIS_LAST_MODIFIED_BY
						.value(), AUTHOR_NAME));
		cpt.getProperty().add(
				getPropertyString(EnumPropertiesBase.CMIS_CREATED_BY.value(),
						AUTHOR_NAME));

		// Uri

		cot.setProperties(cpt);

		if (allowableActions)
			cot.setAllowableActions(getAllowableActions(bot));

		e.getAnyOther().add(coreFactory.createObject(cot));

		switch (bot) {
		case CMIS_DOCUMENT:
			cpt.getProperty().add(
					getPropertyBoolean(
							EnumPropertiesDocument.CMIS_IS_LATEST_VERSION
									.value(), true));
			cpt
					.getProperty()
					.add(
							getPropertyBoolean(
									EnumPropertiesDocument.CMIS_IS_VERSION_SERIES_CHECKED_OUT
											.value(), checkedout));
			cpt.getProperty().add(
					getPropertyBoolean(
							EnumPropertiesDocument.CMIS_IS_MAJOR_VERSION
									.value(), false));
			cpt.getProperty().add(
					getPropertyBoolean(
							EnumPropertiesDocument.CMIS_IS_LATEST_MAJOR_VERSION
									.value(), false));

			cpt.getProperty()
					.add(
							getPropertyString(
									EnumPropertiesDocument.CMIS_CHECKIN_COMMENT
											.value(), "Checkin comment"));
			cpt.getProperty().add(
					getPropertyString(EnumPropertiesDocument.CMIS_VERSION_LABEL
							.value(), "0.1"));
			cpt
					.getProperty()
					.add(
							getPropertyString(
									EnumPropertiesDocument.CMIS_CONTENT_STREAM_MIME_TYPE
											.value(), "text/plain"));
			cpt.getProperty().add(
					getPropertyString(
							EnumPropertiesDocument.CMIS_CONTENT_STREAM_FILENAME
									.value(), "text.txt"));
			cpt.getProperty().add(
					getPropertyInt(
							EnumPropertiesDocument.CMIS_CONTENT_STREAM_LENGTH
									.value(), 4234));

			// provide an MVP for a doc - keywords
			CmisPropertyString cps = new CmisPropertyString();
			cps.setName("keywords");
			cps.getValue().add("document");
			cps.getValue().add("example");
			cps.getValue().add("sample");
			cps.getValue().add("cmis");
			cpt.getProperty().add(cps);

			// <atom:link rel="edit-media"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("edit-media", URL_BASE + objectId
							+ "/edit-media", null, null, null, null));

			// <atom:link rel="alternate"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("alternate",
							URL_BASE + objectId + "/alternate", null, null,
							null, null));

			// <!-- CMIS Links -->
			// <atom:link rel="cmis-parents"
			// href="http://www.cmis.org/rep1/document-entry/parents" />
			e.getLink().add(
					getAtomLink("parents", URL_BASE + objectId + "/parents",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="cmis-allversions"
			// href="http://www.cmis.org/rep1/document-entry/allversions" />
			e.getLink().add(
					getAtomLink("allversions", URL_BASE + objectId
							+ "/allversions", MIMETYPE_ATOMFEED, null, null,
							null));

			// <atom:link rel="cmis-latestversion"
			// href="http://www.cmis.org/rep1/document-entry/latestversion" />
			e.getLink().add(
					getAtomLink("latestversion", URL_BASE + objectId
							+ "/latestversions", MIMETYPE_ATOMENTRY, null,
							null, null));

			// <atom:link rel="cmis-stream"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("stream", URL_BASE + objectId + "media",
							CS_MIMETYPE, CS_LENGTH, null, null));
			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			if (checkedout) {
				cpt
						.getProperty()
						.add(
								getPropertyId(
										EnumPropertiesDocument.CMIS_VERSION_SERIES_CHECKED_OUT_ID
												.value(), "vs-" + objectId));
				cpt
						.getProperty()
						.add(
								getPropertyString(
										EnumPropertiesDocument.CMIS_VERSION_SERIES_CHECKED_OUT_BY
												.value(), AUTHOR_NAME));
				e.getLink().add(
						getAtomLink("pwc", URL_BASE + objectId + "/pwc",
								MIMETYPE_ATOMFEED, null, null, null));

			}

			break;
		case CMIS_FOLDER:
			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			// <atom:link rel="cmis-parent"
			// href="http://www.cmis.org/rep1/folder-entry/parent" />
			e.getLink().add(
					getAtomLink("up", URL_BASE + objectId + "/up",
							MIMETYPE_ATOMENTRY, null, null, null));

			// <cmis:propertyId
			// cmis:name="parentId"><cmis:value>parentFolderId1<
			// /cmis:value></cmis:propertyId>
			cpt.getProperty().add(
					getPropertyId(EnumPropertiesFolder.CMIS_PARENT_ID.value(),
							objectId + "up"));

			// down
			e.getLink().add(
					getAtomLink(EnumLinkRelations.DOWN.value(), URL_BASE
							+ objectId + "/children/flat", MIMETYPE_ATOMFEED,
							null, null, null));

			// down tree
			e.getLink().add(
					getAtomLink(EnumLinkRelations.DOWN.value(), URL_BASE
							+ objectId + "/children/tree", MIMETYPE_CMISTREE,
							null, null, null));

			break;
		case CMIS_POLICY:
			// <!-- CMIS Links -->
			// <atom:link rel="cmis-parents"
			// href="http://www.cmis.org/rep1/document-entry/parents" />
			e.getLink().add(
					getAtomLink("parents", URL_BASE + objectId + "/parents",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			break;
		case CMIS_RELATIONSHIP:
			// <atom:link rel="cmis-source"
			// href="http://www.cmis.org/rep1/policy-entry/source" />
			e.getLink().add(
					getAtomLink("source", URL_BASE + objectId + "/source",
							MIMETYPE_ATOMENTRY, null, null, null));

			// <atom:link rel="cmis-target"
			// href="http://www.cmis.org/rep1/policy-entry/target" />
			e.getLink().add(
					getAtomLink("target", URL_BASE + objectId + "/target",
							MIMETYPE_ATOMENTRY, null, null, null));
			break;
		}

		// Unified search
		if (changeEntry) {
			CmisChangeEventType ccet = new CmisChangeEventType();

			ccet.setChangeType(EnumTypeOfChanges.UPDATED);
			GregorianCalendar cal = new GregorianCalendar();
			cal.setTime(new Date());
			XMLGregorianCalendar xgdt = null;
			try {
				xgdt = DatatypeFactory.newInstance().newXMLGregorianCalendar(
						cal);
			} catch (DatatypeConfigurationException ex) {
				ex.printStackTrace();
			}

			ccet.setChangeTime(xgdt);
			cot.setChangeEventInfo(ccet);
		}

		return e;

	}

	public void test10DocumentEntry() {
		EntryType e = getAtomEntryType(UUID.randomUUID().toString(),
				EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true, false,
				"CMIS Example Document");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntry", element);
	}

	public void test11DocumentEntryPWC() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(),
				EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", true, true, false,
				"CMIS Example Document for PWC");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntryPWC", element);
	}

	public void test12FolderEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(), EnumBaseObjectTypeIds.CMIS_FOLDER,
				"customer", true, true, false,
				"CMIS Example Folder as Customer type");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("FolderEntry", element);
	}

	public void test13PolicyEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(), EnumBaseObjectTypeIds.CMIS_POLICY,
				"policy", true, true, false, "CMIS Example Policy");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("PolicyEntry", element);
	}

	public void test14RelationshipEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(),
				EnumBaseObjectTypeIds.CMIS_RELATIONSHIP, "compounddocument", true,
				true, false,
				"CMIS Example Relationship as Compound Document type");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("RelationshipEntry", element);
	}

	public void test15FolderChildren() {
		FeedType f = getAtomFeedType("folder1feed", "Feed for folder1", true);

		// add a couple of folders
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "invoice", true, true,
						false, "CMIS Example Folder as Invoice type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true,
						false, "CMIS Example Folder as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("FolderChildren", element);
	}

	public void test15FolderDescendants() {
		FeedType f = getAtomFeedType(UUID.randomUUID().toString(),
				"Feed for folder1", false);

		// add a couple of folders
		EntryType e = null;

		// folder1
		String fUUID = UUID.randomUUID().toString();
		e = getAtomEntryType(fUUID,
				EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true, false,
				"CMIS Example Folder as Customer type");

		// folder1 has a couple of docs
		FeedType f1 = getAtomFeedType(fUUID,
				"CMIS Example Folder as Customer type", false);
		
		f1.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));
		f1.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));
		// save
		JAXBElement<FeedType> kids = raFactory.createChildren(f1);
		e.getAnyOther().add(kids);
		f.getEntry().add(e);

		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("FolderDescendants", element);
	}

	private FeedType getAtomFeedType(String id, String title, boolean paging) {
		FeedType f = new FeedType();
		f.setTitle(getAtomText(title, "text"));
		f.getAuthor().add(getAtomPerson(AUTHOR_NAME, AUTHOR_EMAIL, AUTHOR_URI));

		// Set update to now
		f.setUpdated(getAtomDateConstruct(new Date()));

		// set the id
		Id aId = new Id();
		aId.setContent("urn:uuid:" + UUID.randomUUID().toString());
		f.setId(aId);

		// setup app:collection
		AppCollectionType apc = new AppCollectionType();
		apc.setHref(URL_BASE + id + "/3");
		apc.setTitle(getAtomText(title, "text"));

		apc.getAccept().add(MIMETYPE_ATOMENTRY);
		apc.getAccept().add(MIMETYPE_ATOM);
		apc.getAccept().add("*/*");

		// add back to feed
		f.getAnyOther().add(appFactory.createCollection(apc));

		// add self link
		f.getLink().add(
				getAtomLink("self", URL_BASE + id + "/3", MIMETYPE_ATOMFEED,
						null, null, null));

		// add source
		f.getLink().add(
				getAtomLink("via", URL_BASE + id, MIMETYPE_ATOMENTRY, null,
						null, null));

		if (paging) {
			// <!-- RFC 5005 Feed Paging -->
			// <atom:link rel="first"
			// href="http://www.cmis.org/rep1/folder1/children" />
			f.getLink().add(
					getAtomLink("first", URL_BASE + id + "/first",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="next"
			// href="http://www.cmis.org/rep1/folder1/children/4" />
			f.getLink().add(
					getAtomLink("next", URL_BASE + id + "/4",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="previous"
			// href="http://www.cmis.org/rep1/folder1/children/2" />
			f.getLink().add(
					getAtomLink("previous", URL_BASE + id + "/2",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="last"
			// href="http://www.cmis.org/rep1/folder1/children/last" />
			f.getLink().add(
					getAtomLink("last", URL_BASE + id + "/last",
							MIMETYPE_ATOMFEED, null, null, null));

		}

		return f;
	}

	public void test20Query() {
		CmisQueryType q = getQueryType("SELECT * FROM Document", 50, 0, true,
				true);

		// write it
		JAXBElement<CmisQueryType> element = coreFactory.createQuery(q);
		writeOutput("Query", element);
	}

	public void test20Service() {
		AppServiceType q = getServiceType();

		// write it
		JAXBElement<AppServiceType> element = appFactory.createService(q);
		writeOutput("Service", element);
	}

	public void test30DocumentEntryWithChanges() {
		EntryType e = getAtomEntryType(UUID.randomUUID().toString(),
				EnumBaseObjectTypeIds.CMIS_DOCUMENT, "loan", false, true, true,
				"CMIS Example Document - Loan");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntryWithChanges", element);
	}

	public void test40TypeDefinition() {
		JAXBElement<EntryType> element = null;
		EntryType e = null;

		e = getCmisTypeDefinitionType("document-invoice",
				EnumBaseObjectTypeIds.CMIS_DOCUMENT, false);
		element = atomFactory.createEntry(e);
		writeOutput("TypeDocumentWithout", element);

		e = getCmisTypeDefinitionType("document-invoice",
				EnumBaseObjectTypeIds.CMIS_DOCUMENT, true);
		element = atomFactory.createEntry(e);
		writeOutput("TypeDocumentWith", element);

		e = getCmisTypeDefinitionType("folder-invoice",
				EnumBaseObjectTypeIds.CMIS_FOLDER, false);
		element = atomFactory.createEntry(e);
		writeOutput("TypeFolderWithout", element);

		e = getCmisTypeDefinitionType("folder-invoice",
				EnumBaseObjectTypeIds.CMIS_FOLDER, true);
		element = atomFactory.createEntry(e);
		writeOutput("TypeFolderWith", element);

		e = getCmisTypeDefinitionType("customer-relationship",
				EnumBaseObjectTypeIds.CMIS_RELATIONSHIP, false);
		element = atomFactory.createEntry(e);
		writeOutput("TypeRelationshipWithout", element);

		e = getCmisTypeDefinitionType("customer-relationship",
				EnumBaseObjectTypeIds.CMIS_RELATIONSHIP, true);
		element = atomFactory.createEntry(e);
		writeOutput("TypeRelationshipWith", element);

		e = getCmisTypeDefinitionType("security-policy",
				EnumBaseObjectTypeIds.CMIS_POLICY, false);
		element = atomFactory.createEntry(e);
		writeOutput("TypeRelationshipWithout", element);

		e = getCmisTypeDefinitionType("security-policy",
				EnumBaseObjectTypeIds.CMIS_POLICY, true);
		element = atomFactory.createEntry(e);
		writeOutput("TypeRelationshipWith", element);
	}

	public void test30ChangeLog() {
		FeedType f = getAtomFeedType("folder1feed", "changelog feed", true);

		// add a couple of folders
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true,
						true, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "customer", true, true,
						true, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_FOLDER, "invoice", true, true, true,
						"CMIS Example Folder as Invoice type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectTypeIds.CMIS_DOCUMENT, "invoice", false, true,
						true, "CMIS Example Folder as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("ChangeLog", element);
	}

	private static CmisProperty getPropertyDate(String pName, Date date) {
		CmisPropertyDateTime p = new CmisPropertyDateTime();
		p.setName(pName);

		GregorianCalendar c = new GregorianCalendar();
		c.setTime(date);

		try {
			p.getValue().add(
					DatatypeFactory.newInstance().newXMLGregorianCalendar(c));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}

		return p;
	}

	private static CmisProperty getPropertyUri(String pName, String b) {
		CmisPropertyUri p = new CmisPropertyUri();
		p.setName(pName);
		p.getValue().add(b);

		return p;
	}

	private static CmisProperty getPropertyBoolean(String pName, boolean b) {
		CmisPropertyBoolean p = new CmisPropertyBoolean();
		p.setName(pName);
		p.getValue().add(b);

		return p;
	}

	private static CmisProperty getPropertyId(String pName, String v) {
		CmisPropertyId p = new CmisPropertyId();
		p.setName(pName);
		p.getValue().add(v);

		return p;
	}

	private static CmisProperty getPropertyInt(String pName, int v) {
		CmisPropertyInteger p = new CmisPropertyInteger();
		p.setName(pName);
		p.getValue().add(BigInteger.valueOf(v));

		return p;
	}

	private static CmisProperty getPropertyString(String pName, String v) {
		CmisPropertyString p = new CmisPropertyString();
		p.setName(pName);
		p.getValue().add(v);

		return p;
	}

	/**
	 * @author abrown
	 * 
	 * @param text
	 * @param type
	 * @return
	 */
	private static AtomTextConstruct getAtomText(String text, String type) {
		AtomTextConstruct atc = new AtomTextConstruct();
		atc.getContent().add(text);
		atc.setType(type);

		return atc;
	}

	/**
	 * @author abrown
	 * 
	 * @param date
	 * @return
	 */
	private JAXBElement<XMLGregorianCalendar> getAppEdited(Date d) {
		if (d == null) {
			d = new Date();
		}

		GregorianCalendar c = new GregorianCalendar();
		c.setTime(d);

		try {
			return (appFactory.createEdited(DatatypeFactory.newInstance()
					.newXMLGregorianCalendar(c)));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}

		return null;
	}

	private static AtomDateConstruct getAtomDateConstruct(Date d) {
		AtomDateConstruct adc = new AtomDateConstruct();

		if (d == null) {
			return adc;
		}

		GregorianCalendar c = new GregorianCalendar();
		c.setTime(d);

		try {
			adc.setValue(DatatypeFactory.newInstance().newXMLGregorianCalendar(
					c));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}

		return adc;
	}

	private EntryType getCmisTypeDefinitionType(String name,
			EnumBaseObjectTypeIds bot, boolean incPDs) {
		EntryType e = new EntryType();

		String selfUrl = URL_BASE + "type/" + name;
		// <atom:author xml:base="http://tempuri.org" xml:lang="">
		// <atom:name>Al Brown</atom:name>
		// </atom:author>
		e.getAuthor().add(getAtomPerson(AUTHOR_NAME, AUTHOR_EMAIL, AUTHOR_URI));

		// <!-- atom:content -->
		// <atom:content src="http://www.cmis.org/rep1/media/document-entry">
		// </atom:content>

		Content c = new Content();
		c.getContent().add(new String("Type Definition for " + name));
		e.setContent(c);

		// <!-- IRI to document entry resource -->
		// <atom:id>http://www.cmis.org/rep1/document-entry</atom:id>
		Id id = new Id();
		id.setContent(selfUrl);
		e.setId(id);

		// <!-- atom links -->
		// <atom:link rel='self' href="http://www.cmis.org/rep1/document-entry"
		// />
		e.getLink().add(
				getAtomLink(EnumLinkRelations.SELF.value(), selfUrl,
						MIMETYPE_ATOMENTRY, null, null, null));

		// service
		e.getLink().add(
				getAtomLink(EnumLinkRelations.SERVICE.value(), selfUrl,
						MIMETYPE_APPSERVICE, null, null, null));

		// service
		e.getLink().add(
				getAtomLink(EnumLinkRelations.DESCRIBEDBY.value(), URL_BASE
						+ "type/" + bot.value(), MIMETYPE_ATOMENTRY, null,
						null, null));

		// up
		e.getLink().add(
				getAtomLink(EnumLinkRelations.UP.value(), selfUrl + "/parent",
						MIMETYPE_ATOMENTRY, null, null, null));

		// down
		e.getLink()
				.add(
						getAtomLink(EnumLinkRelations.DOWN.value(), selfUrl
								+ "/children/flat", MIMETYPE_ATOMFEED, null,
								null, null));

		// down tree
		e.getLink()
				.add(
						getAtomLink(EnumLinkRelations.DOWN.value(), selfUrl
								+ "/children/tree", MIMETYPE_CMISTREE, null,
								null, null));

		// <!-- createdDate -->
		// <atom:published>2001-12-31T12:00:00</atom:published>
		e.setPublished(getAtomDateConstruct(new Date()));

		// <atom:summary>
		// <!-- auto-generated HTML table of properties -->
		// </atom:summary>
		e.setSummary(getAtomText("HTML summary of Type Definition  " + name,
				"html"));

		// <atom:title>Document Entry example</atom:title>
		e.setTitle(getAtomText("Type Definition - " + name, "text"));

		// <!-- lastModifiedDate -->
		// <atom:updated>2001-12-31T12:00:00</atom:updated>
		e.setUpdated(getAtomDateConstruct(new Date()));

		// add app:edited
		e.getAnyOther().add(getAppEdited(new Date()));

		CmisTypeDefinitionType ctdt = null;

		switch (bot) {
		case CMIS_DOCUMENT:
			CmisTypeDocumentDefinitionType dt = new CmisTypeDocumentDefinitionType();
			ctdt = dt;

			dt.setFileable(true);
			dt.setVersionable(true);
			dt.setContentStreamAllowed(EnumContentStreamAllowed.ALLOWED);

			if (incPDs) {
				// properties
				ctdt.getPropertyDefinition().add(
						getBooleanPropertyDefinition(
								EnumPropertiesDocument.CMIS_IS_IMMUTABLE
										.value(), EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getBooleanPropertyDefinition(
								EnumPropertiesDocument.CMIS_IS_LATEST_VERSION
										.value(), EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getBooleanPropertyDefinition(
								EnumPropertiesDocument.CMIS_IS_MAJOR_VERSION
										.value(), EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getBooleanPropertyDefinition(
										EnumPropertiesDocument.CMIS_IS_LATEST_MAJOR_VERSION
												.value(),
										EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesDocument.CMIS_VERSION_LABEL
										.value(), EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getIdPropertyDefinition(
								EnumPropertiesDocument.CMIS_VERSION_SERIES_ID
										.value(), EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getBooleanPropertyDefinition(
										EnumPropertiesDocument.CMIS_IS_VERSION_SERIES_CHECKED_OUT
												.value(),
										EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getStringPropertyDefinition(
										EnumPropertiesDocument.CMIS_VERSION_SERIES_CHECKED_OUT_BY
												.value(),
										EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getIdPropertyDefinition(
										EnumPropertiesDocument.CMIS_VERSION_SERIES_CHECKED_OUT_ID
												.value(),
										EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesDocument.CMIS_CHECKIN_COMMENT
										.value(), EnumUpdatability.READONLY));

				// cs stream
				ctdt
						.getPropertyDefinition()
						.add(
								getIntegerPropertyDefinition(
										EnumPropertiesDocument.CMIS_CONTENT_STREAM_LENGTH
												.value(),
										EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getStringPropertyDefinition(
										EnumPropertiesDocument.CMIS_CONTENT_STREAM_MIME_TYPE
												.value(),
										EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getStringPropertyDefinition(
										EnumPropertiesDocument.CMIS_CONTENT_STREAM_FILENAME
												.value(),
										EnumUpdatability.READONLY));
			}

			break;
		case CMIS_FOLDER:
			CmisTypeFolderDefinitionType ft = new CmisTypeFolderDefinitionType();
			ctdt = ft;

			if (incPDs) {
				ctdt.getPropertyDefinition().add(
						getIdPropertyDefinition(
								EnumPropertiesFolder.CMIS_PARENT_ID.value(),
								EnumUpdatability.READONLY));
				ctdt
						.getPropertyDefinition()
						.add(
								getIdPropertyDefinition(
										EnumPropertiesFolder.CMIS_ALLOWED_CHILD_OBJECT_TYPE_NAMES
												.value(),
										EnumUpdatability.READWRITE,
										EnumCardinality.MULTI));
			}

			ft.setFileable(true);
			break;
		case CMIS_POLICY:
			CmisTypePolicyDefinitionType pt = new CmisTypePolicyDefinitionType();
			ctdt = pt;

			if (incPDs) {
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesPolicy.CMIS_POLICY_TEXT.value(),
								EnumUpdatability.READWRITE));
			}

			pt.setFileable(false);

			break;
		case CMIS_RELATIONSHIP:
			CmisTypeRelationshipDefinitionType rt = new CmisTypeRelationshipDefinitionType();
			ctdt = rt;

			rt.getAllowedSourceTypes().add("invoice");
			rt.getAllowedSourceTypes().add("capitalinvoice");
			rt.getAllowedTargetTypes().add("customer");

			if (incPDs) {
				ctdt.getPropertyDefinition().add(
						getIdPropertyDefinition(
								EnumPropertiesRelationship.CMIS_SOURCE_ID
										.value(), EnumUpdatability.READWRITE));
				ctdt.getPropertyDefinition().add(
						getIdPropertyDefinition(
								EnumPropertiesRelationship.CMIS_TARGET_ID
										.value(), EnumUpdatability.READWRITE));
			}

			rt.setFileable(false);

			break;
		}

		if (ctdt != null) {
			ctdt.setId("dt" + name);
			ctdt.setLocalName("myrepname-" + name);
			ctdt.setQueryName(name);
			ctdt.setDisplayName(name);
			ctdt.setParentId("parent");
			ctdt.setDescription("Description for type definition " + name);

			// flags
			ctdt.setCreatable(true);

			ctdt.setControllableACL(true);
			ctdt.setControllablePolicy(true);
			ctdt.setIncludedInSupertypeQuery(true);

			if (incPDs) {
				// base properties on everything
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_NAME.value(),
								EnumUpdatability.READWRITE));
				ctdt.getPropertyDefinition().add(
						getIdPropertyDefinition(
								EnumPropertiesBase.CMIS_OBJECT_ID.value(),
								EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_OBJECT_TYPE_ID.value(),
								EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_BASE_TYPE_ID.value(),
								EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_CREATED_BY.value(),
								EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getDatePropertyDefinition(
								EnumPropertiesBase.CMIS_CREATION_DATE.value(),
								EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_LAST_MODIFIED_BY
										.value(), EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getDatePropertyDefinition(
								EnumPropertiesBase.CMIS_LAST_MODIFICATION_DATE
										.value(), EnumUpdatability.READONLY));
				ctdt.getPropertyDefinition().add(
						getStringPropertyDefinition(
								EnumPropertiesBase.CMIS_CHANGE_TOKEN.value(),
								EnumUpdatability.READONLY));
			}

			e.getAnyOther().add(coreFactory.createType(ctdt));
		}
		return e;

	}

	public CmisPropertyStringDefinitionType getStringPropertyDefinition(
			String name, EnumUpdatability eu) {
		CmisPropertyStringDefinitionType ps = new CmisPropertyStringDefinitionType();

		ps.setId(name);
		ps.setLocalName(name);

		ps.setDescription("Description for " + name);
		ps.setDisplayName(name);
		ps.setCardinality(EnumCardinality.SINGLE);
		ps.setInherited(true);
		ps.setMaxLength(BigInteger.valueOf(128));

		ps.setOpenChoice(false);
		ps.setOrderable(true);
		ps.setPropertyType(EnumPropertyType.STRING);
		ps.setQueryable(true);
		ps.setRequired(false);
		ps.setUpdatability(eu);

		return ps;
	}

	public CmisPropertyDateTimeDefinitionType getDatePropertyDefinition(
			String name, EnumUpdatability eu) {
		CmisPropertyDateTimeDefinitionType p = new CmisPropertyDateTimeDefinitionType();

		p.setId(name);
		p.setLocalName(name);

		p.setDescription("Description for " + name);
		p.setDisplayName(name);
		p.setCardinality(EnumCardinality.SINGLE);
		p.setInherited(true);
		p.setOpenChoice(false);
		p.setOrderable(true);
		p.setPropertyType(EnumPropertyType.DATETIME);
		p.setQueryable(true);
		p.setRequired(false);
		p.setUpdatability(eu);

		return p;
	}

	public CmisPropertyIntegerDefinitionType getIntegerPropertyDefinition(
			String name, EnumUpdatability eu) {
		CmisPropertyIntegerDefinitionType p = new CmisPropertyIntegerDefinitionType();

		p.setId(name);
		p.setLocalName(name);

		p.setDescription("Description for " + name);
		p.setDisplayName(name);
		p.setCardinality(EnumCardinality.SINGLE);
		p.setInherited(true);
		p.setOpenChoice(false);
		p.setOrderable(true);
		p.setPropertyType(EnumPropertyType.DATETIME);
		p.setQueryable(true);
		p.setRequired(false);
		p.setUpdatability(eu);

		p.setMaxValue(BigInteger.valueOf(Long.MAX_VALUE));
		p.setMinValue(BigInteger.valueOf(Long.MIN_VALUE));

		return p;
	}

	public CmisPropertyIdDefinitionType getIdPropertyDefinition(String name,
			EnumUpdatability eu) {
		return getIdPropertyDefinition(name, eu, EnumCardinality.SINGLE);
	}

	public CmisPropertyIdDefinitionType getIdPropertyDefinition(String name,
			EnumUpdatability eu, EnumCardinality ec) {
		CmisPropertyIdDefinitionType p = new CmisPropertyIdDefinitionType();

		p.setId(name);
		p.setLocalName(name);

		p.setDescription("Description for " + name);
		p.setDisplayName(name);
		p.setCardinality(ec);
		p.setInherited(true);
		p.setOpenChoice(false);
		p.setOrderable(true);
		p.setPropertyType(EnumPropertyType.ID);
		p.setQueryable(true);
		p.setRequired(false);
		p.setUpdatability(eu);

		return p;
	}

	public CmisPropertyUriDefinitionType getUriPropertyDefinition(String name,
			EnumUpdatability eu) {
		CmisPropertyUriDefinitionType p = new CmisPropertyUriDefinitionType();

		p.setId(name);
		p.setLocalName(name);

		p.setDescription("Description for " + name);
		p.setDisplayName(name);
		p.setCardinality(EnumCardinality.SINGLE);
		p.setInherited(true);
		p.setOpenChoice(false);
		p.setOrderable(true);
		p.setPropertyType(EnumPropertyType.URI);
		p.setQueryable(true);
		p.setRequired(false);
		p.setUpdatability(eu);

		return p;
	}

	public CmisPropertyBooleanDefinitionType getBooleanPropertyDefinition(
			String name, EnumUpdatability eu) {
		CmisPropertyBooleanDefinitionType p = new CmisPropertyBooleanDefinitionType();

		p.setId(name);
		p.setLocalName(name);

		p.setDescription("Description for " + name);
		p.setDisplayName(name);
		p.setCardinality(EnumCardinality.SINGLE);
		p.setInherited(true);
		p.setOpenChoice(false);
		p.setOrderable(true);
		p.setPropertyType(EnumPropertyType.BOOLEAN);
		p.setQueryable(true);
		p.setRequired(false);
		p.setUpdatability(eu);

		return p;
	}

	/**
	 * @author abrown
	 * 
	 * @return
	 */
	private AppServiceType getServiceType() {
		AppServiceType ast = new AppServiceType();
		AppWorkspaceType awt = new AppWorkspaceType();
		ast.getWorkspace().add(awt);

		awt.setTitle(getAtomText("CMIS Repository", "text"));

		awt.getCollection().add(
				getCollectionType(EnumCollectionType.CHECKEDOUT,
						"Checkedout collection", URL_BASE + "/checkedout"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.QUERY, "Query collection",
						URL_BASE + "/query"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.ROOT,
						"Root children collection", URL_BASE + "/rootc"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.TYPES,
						"Types children collection", URL_BASE + "/typec"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.UNFILED,
						"Unfiled collection", URL_BASE + "/unfiled"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.CHANGES,
						"changes collection", URL_BASE + "/changes"));

		Link l = null;

		l = getAtomLink(
				EnumLinkRelations.HTTP_DOCS_OASIS_OPEN_ORG_NS_CMIS_LINK_200901_TYPESDESCENDANTS
						.value(), URL_BASE + "/typesdescendants",
				MIMETYPE_ATOMFEED, null, null, "Types Descendants feed");
		l = getAtomLink(
				EnumLinkRelations.HTTP_DOCS_OASIS_OPEN_ORG_NS_CMIS_LINK_200901_FOLDERTREE
						.value(), URL_BASE + "/foldertree",
				MIMETYPE_ATOMFEED, null, null, "Folder Tree feed for root folder");

		
		awt.getAny().add(l);

		CmisRepositoryInfoType crit = new CmisRepositoryInfoType();

		crit.setCmisVersionSupported(_CMIS_VERSION);
		crit.setProductName("ProductName");
		crit.setProductVersion("0.60");
		crit.setProductName(_PRODUCT_NAME);
		crit.setProductVersion(_PRODUCT_VERSION);
		crit.setRepositoryDescription("CMIS Repository Description");
		crit.setRepositoryId("repid1");
		crit.setRepositoryName("Repository1");
		crit.setRootFolderId("rootfolder");
		crit.setVendorName("CMIS Vendor 1");
		crit.setRepositoryRelationship("self");


		crit.setChangesIncomplete(Boolean.TRUE);
		
		// save it
		awt.getAny().add(raFactory.createRepositoryInfo(crit));

		CmisRepositoryCapabilitiesType crct = new CmisRepositoryCapabilitiesType();
		crct.setCapabilityAllVersionsSearchable(true);
		crct.setCapabilityJoin(EnumCapabilityJoin.INNERANDOUTER);
		crct.setCapabilityMultifiling(true);
		crct.setCapabilityPWCSearchable(true);
		crct.setCapabilityPWCUpdateable(true);
		crct.setCapabilityQuery(EnumCapabilityQuery.BOTHCOMBINED);
		crct.setCapabilityUnfiling(true);
		crct.setCapabilityVersionSpecificFiling(true);
		
		// ACL
		crct.setCapabilityACL(EnumCapabilityACL.MANAGE);
		
		// Content Stream Updates
		crct.setCapabilityContentStreamUpdates(EnumCapabilityContentStreamUpdates.ANYTIME);

		// Unified Search Proposal
		crct.setCapabilityChanges(EnumCapabilityChanges.ALL);
		crct.getCapabilityChangesOnType().add(
				EnumBaseObjectTypeIds.CMIS_DOCUMENT);
		crct.getCapabilityChangesOnType()
				.add(EnumBaseObjectTypeIds.CMIS_FOLDER);
		

		// save
		crit.setCapabilities(crct);

		// add in uri templates
		CmisUriTemplateType cut = new CmisUriTemplateType();
		cut.setMediatype(MIMETYPE_ATOMENTRY);
		cut.setType(EnumUriTemplateType.ENTRYBYID.value());
		cut.setTemplate(URL_BASE + "entrybyid/{id}");
		JAXBElement<CmisUriTemplateType> elUri = raFactory
				.createUritemplate(cut);
		awt.getAny().add(elUri);

		cut = new CmisUriTemplateType();
		cut.setMediatype(MIMETYPE_ATOMENTRY);
		cut.setType(EnumUriTemplateType.FOLDERBYPATH.value());
		cut.setTemplate(URL_BASE + "folderbypath/{path}");
		elUri = raFactory.createUritemplate(cut);
		awt.getAny().add(elUri);

		cut = new CmisUriTemplateType();
		cut.setMediatype(MIMETYPE_ATOMENTRY);
		cut.setType(EnumUriTemplateType.QUERY.value());
		cut
				.setTemplate(URL_BASE
						+ "query?q={path}&searchAllVersions={searchAllVersions}&maxItems={maxItems}&skipCount={skipCount}");
		elUri = raFactory.createUritemplate(cut);
		awt.getAny().add(elUri);

		return ast;
	}

	private static AppCollectionType getCollectionType(
			EnumCollectionType cType, String title, String href) {
		AppCollectionType act = new AppCollectionType();

		act.getOtherAttributes().put(
				new QName("http://docs.oasis-open.org/ns/cmis/restatom/200901",
						"CollectionType"), cType.value());
		// act.setCollectionType(cType);
		act.setHref(href);
		act.setTitle(getAtomText(title, "text"));

		switch (cType) {
		case CHECKEDOUT:
			act.getAccept().add(MIMETYPE_ATOMENTRY);
			break;
		case QUERY:
			act.getAccept().add(MIMETYPE_QUERY);
			break;
		case ROOT:
			break;
		case TYPES:
			break;
		case UNFILED:
			act.getAccept().add(MIMETYPE_ATOMENTRY);
		}

		return act;
	}

	/**
	 * @author abrown
	 * 
	 * @param stmt
	 * @param pageSize
	 * @param skipCount
	 * @param retAllowable
	 * @param retAllVersions
	 * @return
	 */
	private static CmisQueryType getQueryType(String stmt, int pageSize,
			int skipCount, boolean retAllowable, boolean retAllVersions) {
		CmisQueryType cqt = new CmisQueryType();

		cqt.setStatement(stmt);
		cqt.setMaxItems(BigInteger.valueOf(pageSize));
		cqt.setSearchAllVersions(retAllVersions);
		cqt.setSkipCount(BigInteger.valueOf(skipCount));

		return cqt;
	}

	/**
	 * @author abrown
	 * 
	 * @param string
	 * @param string2
	 * @param object
	 * @param object2
	 * @return
	 */
	private static Link getAtomLink(String rel, String href, String type,
			String length, String id, String title) {
		Link l = new Link();
		l.setHref(href);
		l.setRel(rel);
		l.setType(type);
		l.setLength(length);
		l.setTitle(title);

		// add in CMIS specific info (if applicable)
		if (id != null)
			l
					.getOtherAttributes()
					.put(
							new QName(
									"http://docs.oasis-open.org/ns/cmis/restatom/200901",
									"id"), id);
		return l;
	}

	/**
	 * @author abrown
	 * 
	 * @param string
	 * @param string2
	 * @param string3
	 * @return
	 */
	private static AtomPersonConstruct getAtomPerson(String name, String email,
			String uri) {

		AtomPersonConstruct apc = new AtomPersonConstruct();
		apc.setName(name);
		apc.setEmail(email);
		apc.setUri(uri);

		return (apc);
	}

	public static CmisAllowableActionsType getAllowableActions(
			EnumBaseObjectTypeIds ot) {
		CmisAllowableActionsType aa = new CmisAllowableActionsType();

		aa.setCanDelete(true);
		aa.setCanAddObjectToFolder(true);
		aa.setCanApplyPolicy(true);
		aa.setCanGetAppliedPolicies(true);
		aa.setCanGetProperties(true);
		aa.setCanGetRelationships(true);
		aa.setCanMoveObject(true);
		aa.setCanRemoveObjectFromFolder(true);
		aa.setCanRemovePolicy(true);
		aa.setCanUpdateProperties(true);

		switch (ot) {
		case CMIS_DOCUMENT:
			aa.setCanCancelCheckOut(true);
			aa.setCanCheckIn(true);
			aa.setCanCheckOut(true);
			aa.setCanCreateDocument(true);
			aa.setCanDeleteContentStream(true);
			aa.setCanSetContentStream(true);
			aa.setCanGetContentStream(true);
			aa.setCanGetObjectParents(true);
			aa.setCanGetAllVersions(true);
			break;
		case CMIS_FOLDER:
			aa.setCanCreateFolder(true);
			aa.setCanGetChildren(true);
			aa.setCanGetDescendants(true);
			aa.setCanDeleteTree(true);
			aa.setCanGetFolderParent(true);
			break;
		case CMIS_POLICY:
			aa.setCanCreatePolicy(true);
			break;
		case CMIS_RELATIONSHIP:
			aa.setCanCreateRelationship(true);
			break;
		}

		return aa;
	}

	public <T> void writeOutput(String file, JAXBElement<T> element) {
		FileOutputStream out = null;

		try {
			out = new FileOutputStream("examples/" + file + ".xml", false);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}

		if (marshaller == null) {
			getMarshaller();
		}
		try {
			marshaller.marshal(element, out);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
	}

	private static void getMarshaller() {
		try {
			jaxbContext = JAXBContext.newInstance(JAXB_PACKAGE_LIST);
			marshaller = jaxbContext.createMarshaller();

			try {
				marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
				marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
						Boolean.TRUE);

				marshaller.setProperty(
						"com.sun.xml.bind.namespacePrefixMapper",
						new NamespacePrefixMapperImpl());
				marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
						Boolean.TRUE);

			} catch (PropertyException e1) {
				// System.out.println("Property Exception : " + e1.toString());
				e1.printStackTrace();
			}

		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
